{$n+}
{$e+}
unit sio;

(**************************** READ - WRITE ROUTINE *************************)

interface

uses crt,initvar;

procedure filename(var IOfile:string);  {Get filenames}
procedure readspchead;                  {Read spc file head (first 256 bytes)}
procedure readspcdata(var g:rar);       {Read spc data (Y values)}
procedure FYMax(var g:rar);             {Find largest Y value}
procedure ScaleYOut(var g:rar);         {Scale all Y values for storage}
procedure readspc(var g:rar);           {Combines readspchead and readspcdata}
procedure writespc(var g:rar);          {Combines FYMax, ScaleYOut, and write}

implementation

procedure filename(var IOfile:string);

{To read filenames and add extension .spc.  To also check if files exist or
not and let user know about this. Phys is the description of physical meaning of file. Must be defined by
calling routine.}

begin
  write('Enter name of ',phys,' file: (.spc extension assumed) ');
  readln(IOfile);
  IOfile:=IOfile+'.spc';
  if IOfile=infile then
  begin
     {$I-}
     assign(f1,infile);
     reset(f1,256);
     {$I+}
     if IOResult <>0 then                {check if file exists}
     begin
       repeat
        writeln(#007'File does not exist!  Please try again.');
        write('Enter name of ',phys,' file: (.spc extension assumed) ');
        readln(IOfile);
        IOfile:=IOfile+'.spc';
        {$I-}
        assign(f1,infile);
        reset(f1,256);
        {$I+}
       until IOresult=0;
     end; {if IOresult}
  end; {if IOfile=infile}
  if IOfile=outfile then
  begin
    {$I-}
     assign(f2,outfile);
     reset(f2,256);
     {$I+}
     if IOResult=0 then                {check if file exists}
     begin
       repeat
        writeln(#007,outfile,' already exists!!');
        repeat
          write('Would you like to (O)verwrite ',outfile,' or (C)hoose another file name? ');
          readln(ch); ch:=upcase(ch);
        until ch in ['O','C'];
        if ch='C' then
        begin
          write('Enter name of ',phys,' file: (.spc extension assumed) ');
          readln(IOfile);
          IOfile:=IOfile+'.spc';
          {$I-}
          assign(f2,outfile);
          reset(f2,256);
          {$I+}
        end; {if ch=C}
       until (IOresult<>0) or (ch='O');
     end; {if IOresult}
  end; {if IOfile=outfile}
end;  {procedure filename}

procedure readspchead;

 { To read the .spc file header, i.e. the first 256 bytes of an .spc file,
  into array Fhead.  Then to move bytes 3 & 4 to the integer SCE, and bytes
  5 to 8, 9 to 12, and 13 to 16 to the single real variables, PTS, FNU and
  LNU respectively.  SCE is the scaling exponent, PTS is the number of data
  points in the file, FNU is the first wavenumber and LNU is the last
  wavenumber }

begin
  blockread(f1,fhead,1);
  close(f1);
  move(fhead[3],sce,2);   {moves bytes 3,4 into sce as a word}
  move(fhead[5],pts,4);   {moves bytes 5,6,7,8 into pts as single}
  move(fhead[9],fnu,4);   {moves bytes 9,10,11,12 into fnu as single}
  move(fhead[13],lnu,4);  {moves bytes 13,14,15,16 into lnu as single}
  npts:=round(pts);       {npts is now the number of data points in the file}
  delta:=(lnu-fnu)/(npts-1);
end;   {procedure readspchead}

function Convert(w1,w2:word;ex:longint):single;

{ To convert the 2 16-bit words read from an .spc data file by readspcdata
 to a 32-bit longint, to scale the longint by 2^(sce-32) and store in a
 single real variable }

var
   l:longint;
   t1,t2:double;
begin
   l:=w1*(1 shl 16)+w2;  {shift first 2 bytes over 16 bits to left and
                            add next 2 bytes}
   ex:=ex-32;
   t1:=exp(ex*ln(2));
   t2:=l*t1;
   Convert:=t2;
 end; {function Convert}

procedure readspcdata(var g:rar);

{To read .spc ordinate values and convert them (via function CONVERT) to
single real variables }

var
   w1,w2:integer;

begin
   write('reading file.....');
   assign(f3,infile);
   reset(f3);
   seek(f3,0);
   seek(f3,128);
   if not eof(f3) then
   begin
     for i:=1 to npts do
     begin
       read(f3,w1);
       read(f3,w2);
       g[i]:=convert(w1,w2,sce);
     end;  {for i loop}
   end; {if not eof}
   close(f3);
end; {procedure readspcdata}


procedure FYMax(var g:rar);

{ To find the largest value in array g }

begin
  ym:=0;
  for i:=1 to npts do
    if g[i] >= ym then
    begin
      ym:=g[i];
      imax:=i;
    end; {if g[i]}
  writeln('maximum value is : ',ym:5:5,' at point # ',imax);
end;    {procedure FYMax}

procedure ScaleYOut(var g:rar);

{ To find the scaling factor, SCF, and scaling exponent, SCE, and scale
real y for conversion to longint to byte for output to an .spc file.  To find
scaling factor, multiply YMax by 2 until it is just less than the maximum
long integer}

begin
  sc:=0;scf:=1;
  while ym<maxli do
  begin
    ym:=ym*2;
    sc:=sc+1;
    scf:=scf*2;
  end;                {while ym<maxli}
  for i:=1 to npts do
    g[i]:=g[i]*scf;         {Scale all Y values}

  { sc calculated above is 32 minus the sce value.  To be stored in the
  header }

  sce:=32-sc;
end;             {procedure  ScaleYOut}

procedure wrspchd_and_data(var g:rar);

{ To write the spc file header, i.e., the 256 bytes stored in fhead to disk
  and to write the ordinate values to disk as well}

var
  lint:longint;
  hw,lw:word;       {hw = high word, lw = low word}

begin
  write('writing file..... ');
  move(sce,fhead[3],2);   {moves value of sce into bytes 3,4 of fhead}
  move(pts,fhead[5],4);   {moves value of pts into bytes 5,6,7,8 of fhead}
  move(fnu,fhead[9],4);   {moves value of fnu into bytes 9,10,11,12 of fhead}
  move(lnu,fhead[13],4);  {moves value of lnu into bytes 13,14,15,16 of fhead}
  assign(f2,outfile);
  rewrite(f2,256);
  blockwrite(f2,fhead,1);
  i:=1;q:=1;
  for f:=1 to 256 do
     buf[f]:=0;
  for q:=1 to npts do
  begin
    lint:=round(g[q]);
    hw:=wordrec(lint).high;
    lw:=wordrec(lint).low;
    move(hw,buf[i],2);
    move(lw,buf[i+2],2);
    i:=i+4;
    if i>256 then
    begin
      blockwrite(f2,buf,1);
      for f:=1 to 256 do
        buf[f]:=0;
      i:=1;
    end;  {if i>256}
  end; {for q loop}
  blockwrite(f2,buf,1);
  close(f2);
end;           {procedure wrspchd}

procedure readspc(var g:rar);
begin
  readspchead;
  readspcdata(g);
end;  {readspc}

procedure writespc(var g:rar);
begin
  FYMax(g);
  ScaleYOut(g);
  wrspchd_and_data(g);
end;  {writespc}

end. {unit}

(***************************************************************************)





