C       Circlcal.for         
C       circle cell calibration program on PC
C       It uses areas under bands of pATR spectrum
C       May 1, 1992 (second draft)
C       S.L.Zhang, Finalized in June,1992    
C       S.L.Zhang, Modified on July 23, 1992
C               to split the calibration data file for ZnSe rod into 2 files
C                       for spectra recorded with 2 different beamsplitter
C                       i.e. RsZnSeBK for ZnSe rod and KBr beamsplitter
C                       and  RsZnSeBC for ZnSe rod and CaF2 beamsplitter

        real nrf,nrfl,nrfm,nrfr
        logical*1 flagl,flagm,flagr,false,true
        character*4 ext
        character*192 comm
        character*72 prompt
        character*12 inputasc,indata,inspc,inRs,outasc
        dimension areaCl(20), areaCm(20), areaCr(20)
        common/xarray/x(16384)
        common/yarray/y(16384),Rs(16384)
        common/wnarea/wns(20),wne(20),areaE(20),delx,np
        parameter(ird=5,iout=6,maxnp=16384,false=.false.,true=.true.)

        write(*,*)
        write(*,*)' CIRCLCAL, Circle Cell Calibration/PC version 06/92'
        write(*,*)
        write(*,*)' Available calibration data files are:'
        write(*,*)'   RsZnSeBK.cal for ZnSe rod and KBr  beamsplitter'
        write(*,*)'   RsZnSeBC.cal for ZnSe rod and CaF2 beamsplitter'
        write(*,*)'   RsKRS5BK.cal for KRS5 rod and KBr  beamsplitter'
        prompt='\ Enter one of these names (.cal extension assumed):\'
        ext='.cal'
        call getname(prompt,indata,ext,1,ier)

        open (ird,file=indata,status='old')
        read (ird,*) inRs
        read (ird,*) nrf  
        read (ird,*) step1i,step2
        read (ird,*) mode, Nband
        read (ird,*) (wns(i),wne(i),i=1,nband)
        close(ird)
        inquire(file=inRs,exist=flagl)
        if(.not.flagl) then
            write(*,*)' File: ',inRs,' is not in current subdir'
            write(*,*)' or wrong filename in the .cal file entered'
            stop ' Check and re-start'
        endif

10      write(*,*)
        write(*,*)' Input experimental pATR filename (.spc assumed) or'
        prompt=  '\ ASCII filename which supplies pATR filename(s) (.asc
     + assumed):\'
        ext='.spc'
        call getname(prompt,inspc,ext,0,ier)

c       decides where asc data file or spc file input
        inquire (file=inspc,exist=flagl)
        call lenstrz1(inspc,len,ier)
        len1=len+1
        len4=len+4

        if(.not.flagl) goto 20
        nfile=1
        goto 30

c       handles asc file input, 
c       reads experimental pATR filename from the file entered
20      inputasc=inspc
        inputasc(len1:len4)='.asc'
        inquire(file=inputasc,exist=flagl)
        if(.not.flagl) then
                write(*,*)' File: ',inspc,' and ',inputasc,
     +                    ' do not exist -- re-enter'
                goto 10
        endif
        nfile=2
        open (ird,file=inputasc,status='old')

c       gets a filename for ascii output of results
c30      prompt='\ Enter a name for output (.out extension assumed):\'
c        ext='.out'
c        call getname(prompt,outasc,ext,0,ier)
30      outasc(1:len)=inspc(1:len)
        outasc(len1:len4)='.out'

        open (iout,file=outasc,status='unknown')

        if (nfile.eq.1) goto 45

c       reads pATR filename from input ascii data file
44      read(ird,*,end=9999) inspc
        call lenstrz1(inspc,len,ier)
        len1=len+1
        len4=len+4
        inspc(len1:len4)='.spc'

c       make sure that inspc exists
        inquire(file=inspc,exist=flagl)
        if(.not.flagl) then
                write(*,*) 'File: ',inspc,' does not exist.'
                write(iout,*)
                write(iout,*) 'File: ',inspc,' does not exist.'
                goto 44
        endif

45      continue
        write(*,'(a1,a\)')' ','Doing iteration......wait'
        write(iout,9993)
        write(iout,*)'Calibration result for pATR spectrum: ',inspc
        write(iout,*)

c       reads in the experimental pATR and calculates areas
        call readsc(inspc,y,np,xsp,xep,xt,yt,res,comm,ier) 
        call xydata(xsp,xep,np,delx,y)
        call areacalc(xsp,nband,areae,mode)

c       reads in the calculated Rs spectrum 
c       and calculates pATR and areas of the bands
        call readsc(inRs,Rs,np,xsp,xep,xt,yt,res,comm,ier) 
        call xydata(xsp,xep,np,delx,Rs)
        
        flagl=true
        flagm=true
        flagr=true
        iteration=0
        step1=step1i

50      iteration=iteration+1
        nrfl=nrf-step1
        nrfm=nrf
        nrfr=nrf+step1

        if (flagl) call pATRcalc(xsp,nband,nrfl,areacl,mode,suml)
        if (flagm) call pATRcalc(xsp,nband,nrfm,areacm,mode,summ)
        if (flagr) call pATRcalc(xsp,nband,nrfr,areacr,mode,sumr)

c        write(*,*)
c        write(*,*) 'Result for ',inspc
c        write(*,*)'  ','   sumLOW   ','    sumM    ','   sumHIGH  ',
c    +             '       nrfM      increment'      
c        write(*,'(2x,3f12.6,2x,2f10.5,)') suml,summ,sumr,nrf,step1
c        write(*,*) 'nrf=',nrf
c        write(*,9991) (wns(i),' to ',wne(i),areae(i),
c     +      areacm(i),100.*(areacm(i)-areae(i))/areae(i),i=1,nband)
c        write(*,*) 'RMS% deviation=',summ

c        if (summ.gt.suml.and.summ.gt.sumr) then
c       The middle point has maximum deviation!
c                stop 'Something wrong'
        
        if (summ.lt.suml.and.summ.lt.sumr) then
c       The middle point is the minimum of the three
                nrf=nrfm
c               converge limits reached
                if (step1.lt.step2) go to 999
                step1=step1/2.2
                flagl=true
                flagm=false
                flagr=true
                goto 50

        elseif (suml.gt.sumr) then
c       suml > summ > sumr, shift to right one step
        write(iout,'(a,f10.6,a,f9.6)')' RMSPD=',summ,' at NRF=',nrfm
                nrf=nrfr
                suml=summ
                summ=sumr
                flagl=false
                flagm=false
                flagr=true
                do i=1,nband
                   areacl(i)=areacm(i)
                   areacm(i)=areacr(i)
                end do
                goto 50
        else
c       suml < summ < sumr, shift to left one step
        write(iout,'(a,f10.6,a,f9.6)')' RMSPD=',summ,' at NRF=',nrfm
                nrf=nrfl
                sumr=summ
                summ=suml
                flagl=true
                flagm=false
                flagr=false
                do i=1,nband
                   areacr(i)=areacm(i)
                   areacm(i)=areacl(i)
                end do
                goto 50
        endif
        
999     continue        
        write(iout,*)
        write(iout,*) 'Converged number of reflection, NRF=',nrf
        write(iout,9992)
        write(iout,9991) (wns(i),' to ',wne(i),areae(i),
     +      areacm(i),100.*(areacm(i)-areae(i))/areae(i),i=1,nband)
        write(iout,*) 'RMS% deviation=',summ
        write(*,*)
        write(*,*) 'Number of reflection, NRF=',nrf
        write(*,9992)
        write(*,9991) (wns(i),' to ',wne(i),areae(i),
     +      areacm(i),100.*(areacm(i)-areae(i))/areae(i),i=1,nband)
        write(*,*) 'RMS% deviation=',summ
9991    format(f8.2,a,f8.2,2x,1p,2e13.5,0p,f10.2)
9992    format(/,4x,'spectral range ',5x,'areas(Expt)',
     +              '  areas(Calcd)',4x,'%Dev.')
9993    format(1x,//)

        if(nfile.ne.1) goto 44
9999    if(nfile.ne.1) close(iout)
        end

c       prepares x and y array
        subroutine xydata(xsp,xep,np,delx,y)
        common/xarray/x(16384)
        dimension y(*)

c       check if xsp > xep if so reverse the spectrum
        if(xep.gt.xsp)then
                aux=xsp
                xsp=xep
                xep=aux
                do i=1,np/2
                   aux=y(i)
                   y(i)=y(np+1-i)
                   y(np+1-i)=aux
                end do
        end if
c
c       calculate the x data
        delx=(xsp-xep)/(np-1)
        do i=1,np
           x(i)=xsp-delx*(i-1)
        end do
        return
        end

c       Subroutine
        subroutine areacalc(xsp,nband,areax,mode)
        dimension areax(*)
        common/xarray/x(16384)
        common/yarray/y(16384),Rs(16384)
        common/wnarea/wns(20),wne(20),dum(20),delx,np

        do 100 k=1,nband
        wn1=wns(k)
        wn2=wne(k)

c       check if wn1 < wn2 if so then reverse them
        if(wn1.lt.wn2)then
           aux=wn1
           wn1=wn2
           wn2=aux
        end if
c
c       calculate the closest point in x(i) to wn1 and wn2
        np1=(xsp-wn1)/delx+1
        np2=(xsp-wn2)/delx+1
c
c       make sure there is at least two point before wn1 and two after wn2
        if (np1.lt.3) np1=3
        if (np2.gt.np-2) np2=np-2
c
c       caculate avg of five points around wn1 and wn2
        avg1=0.2*(y(np1-2)+y(np1-1)+y(np1)+y(np1+1)+y(np1+2))
        avg2=0.2*(y(np2-2)+y(np2-1)+y(np2)+y(np2+1)+y(np2+2))
        avg=0.5*(avg1+avg2)

c       calculate trap area between wn1 and wn2
        area=0.5*(y(np1)+y(np2))
        do i=np1+1,np2-1
           area=area+y(i)
        end do
        area=area*delx
c
c       substract the baseline if mode =1
        if(mode.gt.0)area=area-(avg*(x(np1)-x(np2)))
100     areax(k)=area
        return
        end

        subroutine pATRcalc(xsp,nband,nrf,areax,mode,sum)
        real nrf
        dimension areax(*)
        common/xarray/x(16384)
        common/yarray/y(16384),Rs(16384)
        common/wnarea/wns(20),wne(20),areae(20),delx,np
        
        do 10 i=1,np
        y(i)=Rs(i)**nrf
10      y(i)=-log10((y(i)*(1.0+y(i)))*0.5)

        call areacalc(xsp,nband,areax,mode)
        sum=0.0
        do 20 i=1,nband
20      sum=sum+((areax(i)-areae(i))/areae(i))**2
        sum=100.0*sqrt(sum/float(nband))

        return
        end

        SUBROUTINE Getname(PROMPT,NAME,ext,IFLAG,IER)
C       THIS ROUTINE PROMPTS FOR A FILENAME AND CHECKS THAT IT DOES
C       EXIST, change or add extension 
        character*4 ext
        CHARACTER*12 NAME
        CHARACTER*72 PROMPT

        LOGICAL FEXIST
        ITR=1
        CALL LENSTR(PROMPT,LEN,IER)
3       WRITE(*,'(2(A)\)') PROMPT(1:LEN),' '
4       READ(*,'(A)',ERR=9001) NAME
        CALL lenstrz1(NAME,LEN,IER)
        LEN1=LEN+1
        LEN4=LEN+4
        NAME(LEN1:LEN4)=ext
C       AVOID ERROR WHEN A BAD NAME IS ENTERED
        INQUIRE(FILE=NAME,EXIST=FEXIST,ERR=9002,IOSTAT=IERR)
 
        IF( IFLAG .EQ. 0 ) RETURN
C       MAKE SURE THE FILE IS THERE.
        INQUIRE(FILE=NAME,EXIST=FEXIST)
        IF( .NOT. FEXIST ) THEN
           IER=0
           WRITE(*,*) ' GETNAME: FILE NOT FOUND - RETYPE.'
           ITR=ITR+1
           IF( ITR .GE.5 ) RETURN
           GO TO 3
        ENDIF
        RETURN
9001    CALL MSG(' AN ERROR WAS ENCOUNTERED READING THAT NAME,')
        CALL MSG(' PLEASE RE-ENTER OR CHOOSE ANOTHER NAME')
        GO TO 3
9002    CALL MSG(' SOMETHING IS WRONG WITH THE FILENAME YOU ENTERED.')
        CALL MSG(' PLEASE CHECK FOR ERRORS OR ILLEGAL CHARACTERS')
        GO TO 3
        END

        SUBROUTINE lenstrz1(ASTR,LEN,IER)
        CHARACTER*100 ASTR

        IER=1
        DO 1 I=2,100
        IF( ASTR(I:I) .EQ.' '.or.astr(I:I).eq.'.' ) THEN
           LEN=I-1
           RETURN
        ENDIF
1       CONTINUE
        IER=-1
        RETURN
        END





